/*****************************************************************************/
/* The development of this program is partly supported by IPA                */
/* (Information-Technology Promotion Agency, Japan).                         */
/*****************************************************************************/

/*****************************************************************************/
/*  bt.h - branch tracer for linux common header                             */
/*  Copyright: Copyright (c) Hitachi, Ltd. 2005-2006                         */
/*             Authors: Yumiko Sugita (sugita@sdl.hitachi.co.jp),            */
/*                      Satoshi Fujiwara (sa-fuji@sdl.hitachi.co.jp)         */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify     */
/*  it under the terms of the GNU General Public License as published by     */
/*  the Free Software Foundation; either version 2 of the License, or        */
/*  (at your option) any later version.                                      */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful,          */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
/*  GNU General Public License for more details.                             */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License        */
/*  along with this program; if not, write to the Free Software              */
/*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA      */
/*****************************************************************************/

#ifndef __BT_H__
#define __BT_H__

#ifndef __KERNEL__
# include <sys/types.h>
#endif

#define MOD_NAME		"bt_mod"
#define BT_VER			"1.0.1"

#define BT_FLAG_T_START		(1 << 26)
#define BT_FLAG_T_STOP		(1 << 27)
#define BT_FLAG_WARN		(1 << 28)
#define BT_FLAG_DEBUG		(1 << 29)
#define BT_FLAG_COMM		(1 << 30)
#define BT_FLAG_PID		(1 << 31)
#define	is_bt_record(r)		\
	(!((r)->flags & (BT_FLAG_WARN|BT_FLAG_DEBUG|BT_FLAG_PID)))
#define	is_pid_record(r)	((r)->flags & BT_FLAG_PID)
#define	is_comm_record(r)	((r)->flags & BT_FLAG_COMM)
#define	is_warn_record(r)	((r)->flags & BT_FLAG_WARN)

#define BT_COMM_LEN		8

struct bt_record {
	unsigned long	from;
	unsigned long	to;
	unsigned long	flags;
};

struct pid_record {
	unsigned long	pid;
	unsigned long	tv_sec;
	unsigned long	tv_usec;
};

struct comm_record {
	char		comm[BT_COMM_LEN];
	unsigned long	flags;
};

struct warn_record {
	unsigned long	left;
	unsigned long	reserve;
	unsigned long	flags;
};

struct tmr_record {
	unsigned long long	timestamp;
	unsigned long		flags;
};

struct ds_manage {
	unsigned long	bts_base;
	unsigned long	bts_index;
	unsigned long	bts_max;
	unsigned long	bts_threshold;
	unsigned long	pebs_base;
	unsigned long	pebs_index;
	unsigned long	pebs_max;
	unsigned long	pebs_threshold;
	unsigned long	pebs_cnt_reset;
	char		reserved[24];
};

struct pid_manage {
	pid_t			pid;
	char			comm[BT_COMM_LEN];
	int			is_wrote;

	/* rdtsc value for checking the syscall execution time */
	long			n_syscall;
	unsigned long long	syscall_start;
	unsigned long long	syscall_end;
};

struct info_per_cpu {
	struct ds_manage	*ds_manage;
	struct pid_manage	pid_manage;
	struct proc_dir_entry	*p_cpuN;
	struct proc_dir_entry	*p_on_off_cnt;
	struct proc_dir_entry	*p_produced;
	struct proc_dir_entry	*p_consumed;
	unsigned long		on_off_cnt;
};

#ifdef __KERNEL__
#include <asm/types.h>

/* MSRs */
#define MSR_IA32_DS_AREA	0x600
#define MSR_DEBUGCTL		0x1d9
struct debugctl_bits {
	int	lbr;
	int	tr;
	int	bts;
	int	btint;
};
#define MSR_DEBUGCTL_P4_BITS	{ 1<<0, 1<<2, 1<<3, 1<<4 }
#define MSR_DEBUGCTL_PM_BITS	{ 1<<0, 1<<6, 1<<7, 1<<8 }

#define	BTS_BUF_MIN_SIZE	(sizeof(struct bt_record) * 64 * 1024)
#define	MIN_INT_MARGIN_RECS	(1 * 1024)
#define	DEFAULT_INT_MARGIN_RECS	(8 * 1024)

/* chk_syscall_time parameter value */
#define CHK_SCTIME_OFF			0
#define CHK_SCTIME_ON			1
#define CHK_SCTIME_ON_WITHOUT_TRACE	2

void bt_enable_per_cpu(void *data);
void bt_disable_per_cpu(void *data);
void bt_enable(void);
void bt_disable(void);

#define is_both(mode)		((mode) == 0)
#define is_start(mode)		((mode) == 1)
#define is_stop(mode)		((mode) == 2)
#define is_fr(mode)  		((mode) == 3)
#define is_syscall_pid(mode) 	((mode) == 4)
#define is_syscall_apid(mode)  	((mode) == 5)
#define is_upid(mode)  		((mode) == 6)

#define is_user(mode) 		((mode) == 6)
#define is_kern(mode)		((mode) < 6)

#define is_kern_pid_by_hook(mode)	(is_both(mode) || is_start(mode))
#define is_kern_all_by_hook(mode)	(is_stop(mode) || is_fr(mode))
#define is_syscall(mode)	(is_syscall_pid(mode) || is_syscall_apid(mode))
#define is_enable_by_proc(mode)	(is_syscall(mode) || is_upid(mode))

/* for serial console prints */
int serial_init(int);
asmlinkage int serial_prints(const char *, ...);

#else

//#define MAX_LINE_LEN	256
#define MAX_LINE_LEN	4096

#endif /* __KERNEL__ */

#endif /*__BT_H__*/
